﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using System.IO;
using wordprocessing;

namespace feature
{
    class Feature
    {
        private EverGrowingDictionary m_dictionary = null;
        private string m_dictionaryFile;
        private double[] m_global = null;
        private double[] m_vector = null;

        public Feature(string dictionaryFile)
        {
            m_dictionaryFile = dictionaryFile;
        }

        public bool populateDictionary()
        {
            Console.WriteLine("Dictionary is being populated ...");
            if (m_dictionary != null) return false;
            if (!File.Exists(m_dictionaryFile))
            {
                Console.WriteLine("Dictionary file not found");
                return false;
            }
            m_dictionary = new EverGrowingDictionary();
            StreamReader dictionaryStream = new StreamReader(m_dictionaryFile);
            string[] s = null;
            while (dictionaryStream.Peek() >= 0)
            {
                s = dictionaryStream.ReadLine().Split(' ');
                if (s.Length != 2)
                {
                    Console.WriteLine("Misformatted dictionary file");
                    return false;
                }
                m_dictionary.GetWordIndex(s[1].Trim());
            }
            dictionaryStream.Close();
            m_dictionary.MakeVocabulary();
            Console.WriteLine("Finished populating dictionary");
            return true;
        }

        public bool readGlobalCount(string fileName)
        {
            FileInfo fi = new FileInfo(fileName);
            if (!fi.Exists) return false;
            long size = fi.Length / 4;
            m_global = new double[size];

            using (FileStream stream = new FileStream(fileName, FileMode.Open))
            {
                using (BinaryReader reader = new BinaryReader(stream))
                {
                    for (long i = 0; i < size; ++i)
                    {
                        int x = reader.ReadInt32();
                        m_global[i] = x;
                    }
                    reader.Close();
                }
                stream.Dispose();
            }
            return true;
        }

        double[] getTestingVector(string dataFile, int[] list)
        {
            FileInfo fi = new FileInfo(dataFile);
            if (!fi.Exists) return null;

            int max = list[0];
            foreach (int m in list)
            {
                if (m > max) max = m;
            }
            ++max;
            int[] chart = new int[max];
            for (int i = 0; i < chart.Length; ++i) chart[i] = 0;
            foreach (int m in list)
            {
                chart[m] = 1;
            }

            long size = fi.Length;
            int nNonZeros = (int)(size / 10);
            m_vector = new double[m_global.Length];
            for (int i = 0; i < m_vector.Length; ++i) m_vector[i] = 0;
            using (FileStream stream = new FileStream(dataFile, FileMode.Open))
            {
                using (BinaryReader reader = new BinaryReader(stream))
                {
                    for (long i = 0; i < nNonZeros; ++i)
                    {
                        int nR = reader.ReadInt32();
                        int nC = reader.ReadInt32();
                        short sum = reader.ReadInt16();
                        if (nR < chart.Length)
                        {
                            if (chart[nR] > 0)
                            {
                                m_vector[nC] += sum;
                            }
                        }
                    }
                    reader.Close();
                }
                stream.Close();
            }
            return m_vector;
        }

        private void orthogonalization()
        {
            double norm = 0.0;
            int size = m_vector.Length;
            for (int i = 0; i < size; ++i)
            {
                norm += m_vector[i] * m_vector[i];
                norm += m_global[i] * m_global[i];
            }
            norm = Math.Sqrt(norm);
            for (int i = 0; i < size; ++i)
            {
                m_vector[i] /= norm;
                m_global[i] /= norm;
            }
            double[,] tmp = new double[2, 2];
            double[] tmp1 = new double[size];
            double[] tmp2 = new double[size];

            int counter = 0;
            while (true)
            {
                tmp[0, 0] = 0.0;
                tmp[0, 1] = 0.0;
                tmp[1, 0] = 0.0;
                tmp[1, 1] = 0.0;
                for (int i = 0; i < size; ++i)
                {
                    tmp[0, 0] += m_vector[i] * m_vector[i];
                    tmp[0, 1] += m_vector[i] * m_global[i];
                    tmp[1, 1] += m_global[i] * m_global[i];
                }
                tmp[1, 0] = tmp[1, 0];

                for (int i = 0; i < size; ++i)
                {
                    tmp1[i] = m_vector[i] * tmp[0, 0] + m_global[i] * tmp[0, 1];
                    tmp2[i] = m_vector[i] * tmp[1, 0] + m_global[i] * tmp[1, 1];
                }

                for (int i = 0; i < size; ++i)
                {
                    m_vector[i] = 1.5 * m_vector[i] - 0.5 * tmp1[i];
                    m_global[i] = 1.5 * m_global[i] - 0.5 * tmp2[i];
                }

                double side = 2.0 * tmp[0, 1];
                double trace = tmp[0, 0] + tmp[1, 1];
                if (side / trace < 0.00001) break;
                if (++counter >= 100) break;
            }
        }

        private byte[] getMask(double coeff)
        {
            byte[] mask = new byte[m_vector.Length];
            double max = m_vector[0];
            for (int i = 0; i < m_vector.Length; ++i)
            {
                if (m_vector[i] > max) max = m_vector[i];
            }

            for (int i = 0; i < m_vector.Length; ++i)
            {
                if (m_vector[i] > max / coeff)
                {
                    mask[i] = 1;

                }
                else
                {
                    mask[i] = 0;
                }
            }
            return mask;
        }

        public string[] getFeature(int[] list, string dataFile, double coeff)
        {
            m_vector = getTestingVector(dataFile, list);
            if (m_vector == null) return null;

            byte[] mask = getMask(coeff);

            for (int i = 0; i < m_vector.Length; ++i)
            {
                m_global[i] -= m_vector[i];
            }
             
            orthogonalization();

            List<double> ff = new List<double>();
            List<int> ii = new List<int>();
            for (int i = 0; i < m_vector.Length; ++i)
            {
                if (mask[i] > 0)
                {
                    double x = m_global[i] / m_vector[i];
                    ff.Add(x);
                    ii.Add(i);
                }
            }

            double[] fff = ff.ToArray();
            int[] iii = ii.ToArray();

            ff.Clear();
            ii.Clear();

            Array.Sort(fff, iii);

            List<string> features = new List<string>();
            for (int i = 0; i < fff.Length; ++i)
            {
                features.Add(m_dictionary.getWord(iii[i]));
            }

            return features.ToArray();
        }
    }
}
