﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
using wordprocessing;

namespace feature
{
    class Program
    {
        static void Main(string[] args)
        {
            string rootFolder = @"..//..//..//..//PATENTCORPUS128";  //This location of data to be processed, make sure it points to your data.
            string extension = "*.txt";
            string dataFolder = "data";  //name of the folder (local to executable) where processed data is stored
            //Next are two names of files with processed data. 
            //ProcessedFilesList.txt is ASCII file with list of found and processed files. 
            string filesList = "ProcessedFilesList.txt";
            string dataFile = "DocWordMatrix.dat";
            string path = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            dataFolder = Path.Combine(path, dataFolder);
            filesList = Path.Combine(dataFolder, filesList);
            dataFile = Path.Combine(dataFolder, dataFile);

            DirectoryInfo di = new DirectoryInfo(rootFolder);
            if (!di.Exists)
            {
                Console.WriteLine("The data folder not found, please correct the path");
                return;
            }

            int N = 128;
            int[] nChart = new int[N];
            for (int i = 0; i < N; ++i)
            {
                nChart[i] = -1;
            }

            DateTime start = DateTime.Now;

            //Here we start word processing and building document-word matrix
            bool useStemming = true;  //This parameter enables or disables stemming.
            WordProcessing wp = new WordProcessing(dataFolder, filesList, dataFile);
            wp.processFiles(rootFolder, extension, nChart, useStemming);
            //At this point document-word matrix is ready.

            Feature feature = new Feature(Path.Combine(dataFolder, "vocabulary.txt"));
            if (!feature.populateDictionary())
            {
                Console.WriteLine("Failed to populate dictionary");
                return;
            }

            if (!feature.readGlobalCount(Path.Combine(dataFolder, "GlobalWordCounter.dat"))) 
            {
                Console.WriteLine("Failed to make global counter vector");
                return;
            }

            int[] list = { 112, 113, 114 };
            double cofficient = 2.5; //this is coeffiecient proportional to the size of the feature array
            string[] ff = feature.getFeature(list, dataFile, cofficient);
            if (ff != null)
            {
                Console.WriteLine("\nExtracted features\n");
                foreach (string s in ff)
                {
                    Console.Write(" {0} ", s);
                }
                Console.WriteLine("\n");
            }
            else
            {
                Console.WriteLine("Failed to extract features");
            }

            DateTime end = DateTime.Now;
            TimeSpan duration = end - start;
            double time = duration.Minutes * 60.0 + duration.Seconds + duration.Milliseconds / 1000.0;
            Console.WriteLine("Total processing time {0:########.00} seconds\n", time);
        }
    }
}


